@@ -230,8 +230,10 @@ divesystem_idive_parser_get_field (dc_parser_t *abstract, dc_field_type_t type,
 				*((dc_divemode_t *) value) = DC_DIVEMODE_OC;
 				break;
 			case SCR:
+				*((dc_divemode_t *) value) = DC_DIVEMODE_SCR;
+				break;
 			case CCR:
-				*((dc_divemode_t *) value) = DC_DIVEMODE_CC;
+				*((dc_divemode_t *) value) = DC_DIVEMODE_CCR;
 				break;
 			case GAUGE:
 				*((dc_divemode_t *) value) = DC_DIVEMODE_GAUGE;
@@ -272,12 +274,14 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 	unsigned int beginpressure = 0;
 	unsigned int endpressure = 0;
 
+	unsigned int firmware = 0;
+	unsigned int apos4 = 0;
 	unsigned int nsamples = array_uint16_le (data + 1);
 	unsigned int samplesize = SZ_SAMPLE_IDIVE;
 	if (parser->model >= IX3M_EASY) {
 		// Detect the APOS4 firmware.
-		unsigned int firmware = array_uint32_le(data + 0x2A);
-		unsigned int apos4 = (firmware / 10000000) >= 4;
+		firmware = array_uint32_le(data + 0x2A);
+		apos4 = (firmware / 10000000) >= 4;
 		if (apos4) {
 			// Dive downloaded and recorded with the APOS4 firmware.
 			samplesize = SZ_SAMPLE_IX3M_APOS4;
@@ -289,6 +293,8 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 			// Dive downloaded and recorded with an older firmware.
 			samplesize = SZ_SAMPLE_IX3M;
 		}
+	} else {
+		firmware = array_uint32_le(data + 0x2E);
 	}
 
 	unsigned int offset = parser->headersize;
@@ -366,17 +372,31 @@ divesystem_idive_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 		}
 
 		// Deco stop / NDL.
-		unsigned int deco = array_uint16_le (data + offset + 21);
-		unsigned int tts  = array_uint16_le (data + offset + 23);
-		if (tts != 0xFFFF) {
-			if (deco) {
+		unsigned int decostop = 0, decotime = 0, tts = 0;
+		if (apos4) {
+			decostop = array_uint16_le (data + offset + 21);
+			decotime = array_uint16_le (data + offset + 23);
+			tts      = array_uint16_le (data + offset + 25);
+			if (tts == 0x7FFF) {
+				tts = INVALID;
+			}
+		} else {
+			decostop = array_uint16_le (data + offset + 21);
+			tts      = array_uint16_le (data + offset + 23);
+			if (tts == 0xFFFF) {
+				tts = INVALID;
+			}
+		}
+		if (tts != INVALID) {
+			if (decostop) {
 				sample.deco.type = DC_DECO_DECOSTOP;
-				sample.deco.depth = deco / 10.0;
+				sample.deco.depth = decostop / 10.0;
+				sample.deco.time = apos4 ? decotime : tts;
 			} else {
 				sample.deco.type = DC_DECO_NDL;
 				sample.deco.depth = 0.0;
+				sample.deco.time = tts;
 			}
-			sample.deco.time = tts;
 			if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
 		}
 
