@@ -47,17 +47,51 @@
 #include "common-private.h"
 #include "context-private.h"
 #include "iostream-private.h"
+#include "iterator-private.h"
+#include "descriptor-private.h"
 #include "array.h"
 #include "platform.h"
 
 #define ISINSTANCE(device) dc_iostream_isinstance((device), &dc_irda_vtable)
 
+#define DISCOVER_MAX_DEVICES 16	// Maximum number of devices.
+#define DISCOVER_MAX_RETRIES 4	// Maximum number of retries.
+
+#ifdef _WIN32
+#define DISCOVER_BUFSIZE sizeof (DEVICELIST) + \
+				sizeof (IRDA_DEVICE_INFO) * (DISCOVER_MAX_DEVICES - 1)
+#else
+#define DISCOVER_BUFSIZE sizeof (struct irda_device_list) + \
+				sizeof (struct irda_device_info) * (DISCOVER_MAX_DEVICES - 1)
+#endif
+
+struct dc_irda_device_t {
+	unsigned int address;
+	unsigned int charset;
+	unsigned int hints;
+	char name[22];
+};
+
 #ifdef IRDA
+static dc_status_t dc_irda_iterator_next (dc_iterator_t *iterator, void *item);
+
+typedef struct dc_irda_iterator_t {
+	dc_iterator_t base;
+	dc_irda_device_t items[DISCOVER_MAX_DEVICES];
+	size_t count;
+	size_t current;
+} dc_irda_iterator_t;
+
+static const dc_iterator_vtable_t dc_irda_iterator_vtable = {
+	sizeof(dc_irda_iterator_t),
+	dc_irda_iterator_next,
+	NULL,
+};
+
 static const dc_iostream_vtable_t dc_irda_vtable = {
 	sizeof(dc_socket_t),
 	dc_socket_set_timeout, /* set_timeout */
 	dc_socket_set_latency, /* set_latency */
-	dc_socket_set_halfduplex, /* set_halfduplex */
 	dc_socket_set_break, /* set_break */
 	dc_socket_set_dtr, /* set_dtr */
 	dc_socket_set_rts, /* set_rts */
@@ -73,73 +107,72 @@ static const dc_iostream_vtable_t dc_irda_vtable = {
 };
 #endif
 
+unsigned int
+dc_irda_device_get_address (dc_irda_device_t *device)
+{
+	if (device == NULL)
+		return 0;
+
+	return device->address;
+}
+
+const char *
+dc_irda_device_get_name (dc_irda_device_t *device)
+{
+	if (device == NULL || device->name[0] == '\0')
+		return NULL;
+
+	return device->name;
+}
+
+void
+dc_irda_device_free (dc_irda_device_t *device)
+{
+	free (device);
+}
+
 dc_status_t
-dc_irda_open (dc_iostream_t **out, dc_context_t *context)
+dc_irda_iterator_new (dc_iterator_t **out, dc_context_t *context, dc_descriptor_t *descriptor)
 {
 #ifdef IRDA
 	dc_status_t status = DC_STATUS_SUCCESS;
-	dc_socket_t *device = NULL;
+	dc_irda_iterator_t *iterator = NULL;
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	// Allocate memory.
-	device = (dc_socket_t *) dc_iostream_allocate (context, &dc_irda_vtable);
-	if (device == NULL) {
+	iterator = (dc_irda_iterator_t *) dc_iterator_allocate (context, &dc_irda_iterator_vtable);
+	if (iterator == NULL) {
 		SYSERROR (context, S_ENOMEM);
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Open the socket.
-	status = dc_socket_open (&device->base, AF_IRDA, SOCK_STREAM, 0);
+	// Initialize the socket library.
+	status = dc_socket_init (context);
 	if (status != DC_STATUS_SUCCESS) {
 		goto error_free;
 	}
 
-	*out = (dc_iostream_t *) device;
-
-    return DC_STATUS_SUCCESS;
-
-error_free:
-	dc_iostream_deallocate ((dc_iostream_t *) device);
-	return status;
-#else
-	return DC_STATUS_UNSUPPORTED;
-#endif
-}
-
-#define DISCOVER_MAX_DEVICES 16	// Maximum number of devices.
-#define DISCOVER_MAX_RETRIES 4	// Maximum number of retries.
-
-#ifdef _WIN32
-#define DISCOVER_BUFSIZE sizeof (DEVICELIST) + \
-				sizeof (IRDA_DEVICE_INFO) * (DISCOVER_MAX_DEVICES - 1)
-#else
-#define DISCOVER_BUFSIZE sizeof (struct irda_device_list) + \
-				sizeof (struct irda_device_info) * (DISCOVER_MAX_DEVICES - 1)
-#endif
-
-dc_status_t
-dc_irda_discover (dc_iostream_t *abstract, dc_irda_callback_t callback, void *userdata)
-{
-#ifdef IRDA
-	dc_socket_t *device = (dc_socket_t *) abstract;
-
-	if (!ISINSTANCE (abstract))
-		return DC_STATUS_INVALIDARGS;
+	// Open the socket.
+	int fd = socket (AF_IRDA, SOCK_STREAM, 0);
+	if (fd == S_INVALID) {
+		s_errcode_t errcode = S_ERRNO;
+		SYSERROR (context, errcode);
+		status = dc_socket_syserror(errcode);
+		goto error_socket_exit;
+	}
 
 	unsigned char data[DISCOVER_BUFSIZE] = {0};
 #ifdef _WIN32
 	DEVICELIST *list = (DEVICELIST *) data;
-	int size = sizeof (data);
 #else
 	struct irda_device_list *list = (struct irda_device_list *) data;
-	socklen_t size = sizeof (data);
 #endif
+	s_socklen_t size = sizeof (data);
 
 	int rc = 0;
 	unsigned int nretries = 0;
-	while ((rc = getsockopt (device->fd, SOL_IRLMP, IRLMP_ENUMDEVICES, (char*) data, &size)) != 0 ||
+	while ((rc = getsockopt (fd, SOL_IRLMP, IRLMP_ENUMDEVICES, (char *) data, &size)) != 0 ||
 #ifdef _WIN32
 		list->numDevice == 0)
 #else
@@ -153,14 +186,16 @@ dc_irda_discover (dc_iostream_t *abstract, dc_irda_callback_t callback, void *us
 		if (rc != 0) {
 			s_errcode_t errcode = S_ERRNO;
 			if (errcode != S_EAGAIN) {
-				SYSERROR (abstract->context, errcode);
-				return dc_socket_syserror(errcode);
+				SYSERROR (context, errcode);
+				status = dc_socket_syserror(errcode);
+				goto error_socket_close;
 			}
 		}
 
 		// Abort if the maximum number of retries is reached.
-		if (nretries++ >= DISCOVER_MAX_RETRIES)
-			return DC_STATUS_SUCCESS;
+		if (nretries++ >= DISCOVER_MAX_RETRIES) {
+			break;
+		}
 
 		// Restore the size parameter in case it was
 		// modified by the previous getsockopt call.
@@ -173,88 +208,109 @@ dc_irda_discover (dc_iostream_t *abstract, dc_irda_callback_t callback, void *us
 #endif
 	}
 
-	if (callback) {
+	S_CLOSE (fd);
+	dc_socket_exit (context);
+
+	dc_filter_t filter = dc_descriptor_get_filter (descriptor);
+
+	unsigned int count = 0;
 #ifdef _WIN32
-		for (unsigned int i = 0; i < list->numDevice; ++i) {
-			const char *name = list->Device[i].irdaDeviceName;
-			unsigned int address = array_uint32_le (list->Device[i].irdaDeviceID);
-			unsigned int charset = list->Device[i].irdaCharSet;
-			unsigned int hints = (list->Device[i].irdaDeviceHints1 << 8) +
-									list->Device[i].irdaDeviceHints2;
+	for (size_t i = 0; i < list->numDevice; ++i) {
+		const char *name = list->Device[i].irdaDeviceName;
+		unsigned int address = array_uint32_le (list->Device[i].irdaDeviceID);
+		unsigned int charset = list->Device[i].irdaCharSet;
+		unsigned int hints = (list->Device[i].irdaDeviceHints1 << 8) +
+								list->Device[i].irdaDeviceHints2;
 #else
-		for (unsigned int i = 0; i < list->len; ++i) {
-			const char *name = list->dev[i].info;
-			unsigned int address = list->dev[i].daddr;
-			unsigned int charset = list->dev[i].charset;
-			unsigned int hints = array_uint16_be (list->dev[i].hints);
+	for (size_t i = 0; i < list->len; ++i) {
+		const char *name = list->dev[i].info;
+		unsigned int address = list->dev[i].daddr;
+		unsigned int charset = list->dev[i].charset;
+		unsigned int hints = array_uint16_be (list->dev[i].hints);
 #endif
 
-			INFO (abstract->context,
-				"Discover: address=%08x, name=%s, charset=%02x, hints=%04x",
-				address, name, charset, hints);
+		INFO (context, "Discover: address=%08x, name=%s, charset=%02x, hints=%04x",
+			address, name, charset, hints);
 
-			callback (address, name, charset, hints, userdata);
+		if (filter && !filter (DC_TRANSPORT_IRDA, name)) {
+			continue;
 		}
+
+		strncpy(iterator->items[count].name, name, sizeof(iterator->items[count].name) - 1);
+		iterator->items[count].name[sizeof(iterator->items[count].name) - 1] = '\0';
+		iterator->items[count].address = address;
+		iterator->items[count].charset = charset;
+		iterator->items[count].hints = hints;
+		count++;
 	}
 
+	iterator->current = 0;
+	iterator->count = count;
+
+	*out = (dc_iterator_t *) iterator;
+
 	return DC_STATUS_SUCCESS;
+
+error_socket_close:
+	S_CLOSE (fd);
+error_socket_exit:
+	dc_socket_exit (context);
+error_free:
+	dc_iterator_deallocate ((dc_iterator_t *) iterator);
+	return status;
 #else
 	return DC_STATUS_UNSUPPORTED;
 #endif
 }
 
-dc_status_t
-dc_irda_connect_name (dc_iostream_t *abstract, unsigned int address, const char *name)
-{
 #ifdef IRDA
-	dc_socket_t *device = (dc_socket_t *) abstract;
+static dc_status_t
+dc_irda_iterator_next (dc_iterator_t *abstract, void *out)
+{
+	dc_irda_iterator_t *iterator = (dc_irda_iterator_t *) abstract;
+	dc_irda_device_t *device = NULL;
 
-	if (!ISINSTANCE (abstract))
-		return DC_STATUS_INVALIDARGS;
+	if (iterator->current >= iterator->count)
+		return DC_STATUS_DONE;
 
-	INFO (abstract->context, "Connect: address=%08x, name=%s", address, name ? name : "");
-
-#ifdef _WIN32
-	SOCKADDR_IRDA peer;
-	peer.irdaAddressFamily = AF_IRDA;
-	peer.irdaDeviceID[0] = (address      ) & 0xFF;
-	peer.irdaDeviceID[1] = (address >>  8) & 0xFF;
-	peer.irdaDeviceID[2] = (address >> 16) & 0xFF;
-	peer.irdaDeviceID[3] = (address >> 24) & 0xFF;
-	if (name) {
-		strncpy (peer.irdaServiceName, name, sizeof(peer.irdaServiceName) - 1);
-		peer.irdaServiceName[sizeof(peer.irdaServiceName) - 1] = '\0';
-	} else {
-		memset (peer.irdaServiceName, 0x00, sizeof(peer.irdaServiceName));
+	device = (dc_irda_device_t *) malloc (sizeof(dc_irda_device_t));
+	if (device == NULL) {
+		SYSERROR (abstract->context, S_ENOMEM);
+		return DC_STATUS_NOMEMORY;
 	}
-#else
-	struct sockaddr_irda peer;
-	peer.sir_family = AF_IRDA;
-	peer.sir_addr = address;
-	if (name) {
-		strncpy (peer.sir_name, name, sizeof(peer.sir_name) - 1);
-		peer.sir_name[sizeof(peer.sir_name) - 1] = '\0';
-	} else {
-		memset (peer.sir_name, 0x00, sizeof(peer.sir_name));
-	}
-#endif
 
-	return dc_socket_connect (&device->base, (struct sockaddr *) &peer, sizeof (peer));
-#else
-	return DC_STATUS_UNSUPPORTED;
-#endif
+	*device = iterator->items[iterator->current++];
+
+	*(dc_irda_device_t **) out = device;
+
+	return DC_STATUS_SUCCESS;
 }
+#endif
 
 dc_status_t
-dc_irda_connect_lsap (dc_iostream_t *abstract, unsigned int address, unsigned int lsap)
+dc_irda_open (dc_iostream_t **out, dc_context_t *context, unsigned int address, unsigned int lsap)
 {
 #ifdef IRDA
-	dc_socket_t *device = (dc_socket_t *) abstract;
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_socket_t *device = NULL;
 
-	if (!ISINSTANCE (abstract))
+	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	INFO (abstract->context, "Connect: address=%08x, lsap=%u", address, lsap);
+	INFO (context, "Open: address=%08x, lsap=%u", address, lsap);
+
+	// Allocate memory.
+	device = (dc_socket_t *) dc_iostream_allocate (context, &dc_irda_vtable);
+	if (device == NULL) {
+		SYSERROR (context, S_ENOMEM);
+		return DC_STATUS_NOMEMORY;
+	}
+
+	// Open the socket.
+	status = dc_socket_open (&device->base, AF_IRDA, SOCK_STREAM, 0);
+	if (status != DC_STATUS_SUCCESS) {
+		goto error_free;
+	}
 
 #ifdef _WIN32
 	SOCKADDR_IRDA peer;
@@ -272,7 +328,20 @@ dc_irda_connect_lsap (dc_iostream_t *abstract, unsigned int address, unsigned in
 	memset (peer.sir_name, 0x00, sizeof(peer.sir_name));
 #endif
 
-	return dc_socket_connect (&device->base, (struct sockaddr *) &peer, sizeof (peer));
+	status = dc_socket_connect (&device->base, (struct sockaddr *) &peer, sizeof (peer));
+	if (status != DC_STATUS_SUCCESS) {
+		goto error_close;
+	}
+
+	*out = (dc_iostream_t *) device;
+
+	return DC_STATUS_SUCCESS;
+
+error_close:
+	dc_socket_close (&device->base);
+error_free:
+	dc_iostream_deallocate ((dc_iostream_t *) device);
+	return status;
 #else
 	return DC_STATUS_UNSUPPORTED;
 #endif
