@@ -195,7 +195,7 @@ dc_parser_new2 (dc_parser_t **out, dc_context_t *context, dc_descriptor_t *descr
 	return dc_parser_new_internal (out, context,
 		dc_descriptor_get_type (descriptor),
 		dc_descriptor_get_model (descriptor),
-		dc_descriptor_get_serial (descriptor),
+		0,
 		devtime, systime);
 }
 
