@@ -143,9 +143,8 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_solution_device_t *device = (suunto_solution_device_t*) abstract;
 
-	// Erase the current contents of the buffer and
-	// allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, SZ_MEMORY)) {
+	// Allocate the required amount of memory.
+	if (!dc_buffer_resize (buffer, SZ_MEMORY)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
 		return DC_STATUS_NOMEMORY;
 	}
@@ -161,7 +160,11 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	unsigned char answer[3] = {0};
 
 	// Assert DTR
-	dc_iostream_set_dtr (device->iostream, 1);
+	status = dc_iostream_set_dtr(device->iostream, 1);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to set the DTR line.");
+		return status;
+	}
 
 	// Send: 0xFF
 	command[0] = 0xFF;
