@@ -54,6 +54,8 @@
 #include "common-private.h"
 #include "context-private.h"
 #include "iostream-private.h"
+#include "descriptor-private.h"
+#include "iterator-private.h"
 #include "platform.h"
 
 #ifdef _WIN32
@@ -66,12 +68,31 @@ typedef pthread_mutex_t dc_mutex_t;
 
 #define ISINSTANCE(device) dc_iostream_isinstance((device), &dc_usbhid_vtable)
 
+struct dc_usbhid_device_t {
+	unsigned short vid, pid;
+};
+
 #ifdef USBHID
+static dc_status_t dc_usbhid_iterator_next (dc_iterator_t *iterator, void *item);
+static dc_status_t dc_usbhid_iterator_free (dc_iterator_t *iterator);
+
 static dc_status_t dc_usbhid_set_timeout (dc_iostream_t *iostream, int timeout);
 static dc_status_t dc_usbhid_read (dc_iostream_t *iostream, void *data, size_t size, size_t *actual);
 static dc_status_t dc_usbhid_write (dc_iostream_t *iostream, const void *data, size_t size, size_t *actual);
 static dc_status_t dc_usbhid_close (dc_iostream_t *iostream);
 
+typedef struct dc_usbhid_iterator_t {
+	dc_iterator_t base;
+	dc_filter_t filter;
+#if defined(USE_LIBUSB)
+	struct libusb_device **devices;
+	size_t count;
+	size_t current;
+#elif defined(USE_HIDAPI)
+	struct hid_device_info *devices, *current;
+#endif
+} dc_usbhid_iterator_t;
+
 typedef struct dc_usbhid_t {
 	/* Base class. */
 	dc_iostream_t base;
@@ -88,11 +109,16 @@ typedef struct dc_usbhid_t {
 #endif
 } dc_usbhid_t;
 
+static const dc_iterator_vtable_t dc_usbhid_iterator_vtable = {
+	sizeof(dc_usbhid_iterator_t),
+	dc_usbhid_iterator_next,
+	dc_usbhid_iterator_free,
+};
+
 static const dc_iostream_vtable_t dc_usbhid_vtable = {
 	sizeof(dc_usbhid_t),
 	dc_usbhid_set_timeout, /* set_timeout */
 	NULL, /* set_latency */
-	NULL, /* set_halfduplex */
 	NULL, /* set_break */
 	NULL, /* set_dtr */
 	NULL, /* set_rts */
@@ -286,6 +312,229 @@ dc_usbhid_exit (void)
 }
 #endif
 
+unsigned int
+dc_usbhid_device_get_vid (dc_usbhid_device_t *device)
+{
+	if (device == NULL)
+		return 0;
+
+	return device->vid;
+}
+
+unsigned int
+dc_usbhid_device_get_pid (dc_usbhid_device_t *device)
+{
+	if (device == NULL)
+		return 0;
+
+	return device->pid;
+}
+
+void
+dc_usbhid_device_free(dc_usbhid_device_t *device)
+{
+	free (device);
+}
+
+dc_status_t
+dc_usbhid_iterator_new (dc_iterator_t **out, dc_context_t *context, dc_descriptor_t *descriptor)
+{
+#ifdef USBHID
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_usbhid_iterator_t *iterator = NULL;
+
+	if (out == NULL)
+		return DC_STATUS_INVALIDARGS;
+
+	iterator = (dc_usbhid_iterator_t *) dc_iterator_allocate (context, &dc_usbhid_iterator_vtable);
+	if (iterator == NULL) {
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
+	}
+
+	// Initialize the usb library.
+	status = dc_usbhid_init (context);
+	if (status != DC_STATUS_SUCCESS) {
+		goto error_free;
+	}
+
+#if defined(USE_LIBUSB)
+	// Enumerate the USB devices.
+	struct libusb_device **devices = NULL;
+	ssize_t ndevices = libusb_get_device_list (g_usbhid_ctx, &devices);
+	if (ndevices < 0) {
+		ERROR (context, "Failed to enumerate the usb devices (%s).",
+			libusb_error_name (ndevices));
+		status = syserror (ndevices);
+		goto error_usb_exit;
+	}
+
+	iterator->devices = devices;
+	iterator->count = ndevices;
+	iterator->current = 0;
+#elif defined(USE_HIDAPI)
+	struct hid_device_info *devices = hid_enumerate(0x0, 0x0);
+	if (devices == NULL) {
+		status = DC_STATUS_IO;
+		goto error_usb_exit;
+	}
+
+	iterator->devices = devices;
+	iterator->current = devices;
+#endif
+	iterator->filter = dc_descriptor_get_filter (descriptor);
+
+	*out = (dc_iterator_t *) iterator;
+
+	return DC_STATUS_SUCCESS;
+
+error_usb_exit:
+	dc_usbhid_exit ();
+error_free:
+	dc_iterator_deallocate ((dc_iterator_t *) iterator);
+	return status;
+#else
+	return DC_STATUS_UNSUPPORTED;
+#endif
+}
+
+#ifdef USBHID
+static dc_status_t
+dc_usbhid_iterator_next (dc_iterator_t *abstract, void *out)
+{
+	dc_usbhid_iterator_t *iterator = (dc_usbhid_iterator_t *) abstract;
+	dc_usbhid_device_t *device = NULL;
+
+#if defined(USE_LIBUSB)
+	while (iterator->current < iterator->count) {
+		struct libusb_device *current = iterator->devices[iterator->current++];
+
+		// Get the device descriptor.
+		struct libusb_device_descriptor dev;
+		int rc = libusb_get_device_descriptor (current, &dev);
+		if (rc < 0) {
+			ERROR (abstract->context, "Failed to get the device descriptor (%s).",
+				libusb_error_name (rc));
+			return syserror (rc);
+		}
+
+		dc_usb_desc_t usb = {dev.idVendor, dev.idProduct};
+		if (iterator->filter && !iterator->filter (DC_TRANSPORT_USBHID, &usb)) {
+			continue;
+		}
+
+		// Get the active configuration descriptor.
+		struct libusb_config_descriptor *config = NULL;
+		rc = libusb_get_active_config_descriptor (current, &config);
+		if (rc != LIBUSB_SUCCESS) {
+			ERROR (abstract->context, "Failed to get the configuration descriptor (%s).",
+				libusb_error_name (rc));
+			return syserror (rc);
+		}
+
+		// Find the first HID interface.
+		const struct libusb_interface_descriptor *interface = NULL;
+		for (unsigned int i = 0; i < config->bNumInterfaces; i++) {
+			const struct libusb_interface *iface = &config->interface[i];
+			for (int j = 0; j < iface->num_altsetting; j++) {
+				const struct libusb_interface_descriptor *desc = &iface->altsetting[j];
+				if (desc->bInterfaceClass == LIBUSB_CLASS_HID && interface == NULL) {
+					interface = desc;
+				}
+			}
+		}
+
+		if (interface == NULL) {
+			libusb_free_config_descriptor (config);
+			continue;
+		}
+
+		// Find the first input and output interrupt endpoints.
+		const struct libusb_endpoint_descriptor *ep_in = NULL, *ep_out = NULL;
+		for (unsigned int i = 0; i < interface->bNumEndpoints; i++) {
+			const struct libusb_endpoint_descriptor *desc = &interface->endpoint[i];
+
+			unsigned int type = desc->bmAttributes & LIBUSB_TRANSFER_TYPE_MASK;
+			unsigned int direction = desc->bEndpointAddress & LIBUSB_ENDPOINT_DIR_MASK;
+
+			if (type != LIBUSB_TRANSFER_TYPE_INTERRUPT) {
+				continue;
+			}
+
+			if (direction == LIBUSB_ENDPOINT_IN && ep_in == NULL) {
+				ep_in = desc;
+			}
+
+			if (direction == LIBUSB_ENDPOINT_OUT && ep_out == NULL) {
+				ep_out = desc;
+			}
+		}
+
+		if (ep_in == NULL || ep_out == NULL) {
+			libusb_free_config_descriptor (config);
+			continue;
+		}
+
+		device = (dc_usbhid_device_t *) malloc (sizeof(dc_usbhid_device_t));
+		if (device == NULL) {
+			ERROR (abstract->context, "Failed to allocate memory.");
+			libusb_free_config_descriptor (config);
+			return DC_STATUS_NOMEMORY;
+		}
+
+		device->vid = dev.idVendor;
+		device->pid = dev.idProduct;
+
+		*(dc_usbhid_device_t **) out = device;
+
+		libusb_free_config_descriptor (config);
+
+		return DC_STATUS_SUCCESS;
+	}
+#elif defined(USE_HIDAPI)
+	while (iterator->current) {
+		struct hid_device_info *current = iterator->current;
+		iterator->current = current->next;
+
+		dc_usb_desc_t usb = {current->vendor_id, current->product_id};
+		if (iterator->filter && !iterator->filter (DC_TRANSPORT_USBHID, &usb)) {
+			continue;
+		}
+
+		device = (dc_usbhid_device_t *) malloc (sizeof(dc_usbhid_device_t));
+		if (device == NULL) {
+			ERROR (abstract->context, "Failed to allocate memory.");
+			return DC_STATUS_NOMEMORY;
+		}
+
+		device->vid = current->vendor_id;
+		device->pid = current->product_id;
+
+		*(dc_usbhid_device_t **) out = device;
+
+		return DC_STATUS_SUCCESS;
+	}
+#endif
+
+	return DC_STATUS_DONE;
+}
+
+static dc_status_t
+dc_usbhid_iterator_free (dc_iterator_t *abstract)
+{
+	dc_usbhid_iterator_t *iterator = (dc_usbhid_iterator_t *) abstract;
+
+#if defined(USE_LIBUSB)
+	libusb_free_device_list (iterator->devices, 1);
+#elif defined(USE_HIDAPI)
+	hid_free_enumeration (iterator->devices);
+#endif
+	dc_usbhid_exit ();
+
+	return DC_STATUS_SUCCESS;
+}
+#endif
+
 dc_status_t
 dc_usbhid_open (dc_iostream_t **out, dc_context_t *context, unsigned int vid, unsigned int pid)
 {
@@ -599,4 +848,12 @@ out:
 
 	return status;
 }
+#else /* !USBHID */
+
+dc_status_t
+dc_usbhid_custom_io (dc_context_t *context, unsigned int vid, unsigned int pid)
+{
+	return DC_STATUS_UNSUPPORTED;
+}
+
 #endif
